/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

interface nsIVariant;

[scriptable, builtinclass, uuid(f0def4a3-0eb3-4c08-b476-582e37b8564b)]
interface nsIDOMPopStateEvent : nsIDOMEvent
{
  /**
   * The state associated with this popstate event
   */
  readonly attribute nsIVariant state;

  void initPopStateEvent(in DOMString typeArg,
                         in boolean canBubbleArg,
                         in boolean cancelableArg,
                         in nsIVariant stateArg);
};

dictionary PopStateEventInit : EventInit
{
  nsIVariant state;
};
