/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=2 sw=2 et tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
#include "domstubs.idl"

[scriptable, uuid(0ccbcf19-d1b4-489e-984c-cd8c43672bb9)]
interface nsIDOMDOMConstructor : nsISupports
{
  AString toString();
};

