/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/*
 * nsslocks.h - threadsafe functions to initialize lock pointers.
 *
 * NOTE - The interfaces formerly in this header were private and are now all
 *        obsolete.
 */
